#pragma once
#include <string>

/*
 * © 2012, Lewis E. Johnson
 * Developed by Lewis E. Johnson and Robin Barnes, Robinson Group, University of Washington.
 * Released under University of Illinois/NCSA Open Source License. See LICENSE.txt for details.
 */

/*This increasingly inaccurately-named class used to just store contants and compile-time flags,
 but evolved into an inelegant but functional method of storing any value needed by multiple
 classes or functions. The .cpp file does not contain anything, and the header only holds variable
 declarations; all of these variables are assigned values elsewhere, generally Main.ccp or Setup.cpp.
 Variables that can changed at runtime are tagged with RUNTIME, variables that are auto-set by Setup
 are tagged with SETUP, and dead code is tagged with DEPRECATED.
 
 Last updated by LEJ 01/15/2011*/

using namespace std;

class Cnsts
{
public:
	Cnsts(void);
public:
	~Cnsts(void);
	
	//RNG memory. Must not be constant! (RUNTIME)
	long rngseed;
	long* idum;	
	
	
	//Batch job data
	int runs;	//number of runs in a batch
	int simnum;	//current run (RUNTIME)
	string fileout; //Root of name for output file
	string filein;	//Root of name for coordinate input file
	string chkpt;	//Identifier (e.g. final) for checkpoint file
	
	//Boolean flags for configuring simulation type
	bool test;	//Enables test mode if true
	bool cut;	//Enables potential truncation if true
	bool PBCs[3];	//Enables x,y,z periodic boundary conditions
	bool dogr;	//Enables calculation of correlation functions if true
	bool scrf;	//Enables SCRF if true
	bool chkcoord;	//Dumps intermediate coordinate files if true
	bool touchtrunc;	//Uses an additional, smaller truncation radius for steric anisotropy if true 
	bool doxm;	//Calculates extended order parameters (wrt x,y,z) if true
	bool rot_notransA;	//Type A molecules cannot translate if true
	bool stillmembersA;	//Type A molecules cannot translate or rotate if true
	bool rot_notransB;	//Same as above, but type B
	bool stillmembersB;	//ditto
	bool averageA;	//Type A molecules are included in order parameter calculations if true
	bool averageB;	//Same, but type B
	bool offctrmu;	//Molecular dipoles are allowed to be off-center if true (SLOW!)
	bool realfield;	//Poling field defined at constant potential (Maxwell field) if true
	bool anneal;	//Simulated annealing used if true
	bool polarize;	//Dipole moments enhanced by Lorentz cavity factor if true
	bool binaryavg;	//Calculate separate <cos^n> for A and B

	//Integer parameters - none of these should be adjusted during run
	int vdwtype;	//Sets LJ interaction type
	int loader;	//Sets loading algorithm used for building the box
	int LJexp[2];	//Sets Lennard-Jones exponents
	int latticetype;	//Sets lattice type for initial box/sphere
	int n_oids;	//Total number of molecules in system
	int n_avg;	//Total number of molecules being averaged (SETUP)
	int lin_dim[3];	//Dimensions of initial lattice in particles (product must == noids)
	int num_A;	//Number of type A particles
	int num_B;	//Number of type B particles
	int xmfreq;	//Frequency of extended means calculations
	int grinc;	//Number of grid elements used for discretizing correlation functions
	int grfreq;	//Frequency for correlation function calculations
	long steps;	//Total steps in a simulation
	long randsteps;	//Number of steps used for randomization at beginning of simulations
	long avg_steps;	//Number of steps used for averaging at end of simulation

	//Parameters that may be adjusted during run (most currently aren't adjustable, though)
	double epsilon;	//Initial reaction field dielectric constant (dynamic value is NOT stored here)
	double boxlength[3];	//Length of the x,y,z dimensions of the box
	double spherecylr;	//Radius if a spherical or cylinderical box is used
	double V;	//Volume of box
	double rmax;	//Cutoff distance for correlation functions
	double refdr;	//Size of each grid element for correlation functions
	double maxtrans;	//Maximum distance for a translation move
	double maxrot;	//Maximum angle for a rotational move
	double rfcorr; //Precalculated reaction field strength, regularly updated (RUNTIME)
	double n2;	//Square of refractive index, used for implicit polarizability
	double in2;	//Inverse of square of refractive index (SETUP)
	double mucorr;	//Lorentz field fractor for scaling dipole moments (SETUP)
	double Rand_Time; // Time the randomization step took
	
	//dynamic E and T adjustment
	int trackchange;	//Counter for temperature changes, currently always 0
	int changesteps;	//Setps at which T is changed, not currently used
	double Efield;	//Initial poling field strength. How this is defined depends on cnsts->realfield
	double T;	//Initial absolute temperature
	double kT;	//self-explanatory
	double beta; //as in 1/kT
	
	//Parameters that should not be adjusted during run
	double kB;	//Boltzmann's constnat
	double ec;	//Charge conversion factor
	double ergc;	//Converts ergs to pico-ergs
	double perg2Jmol;	//Converts pico-ergs to J/mol
	double Escale;	//Converts MV/m into pico-ergs/Debye
	double r0A[3];	//vdw sigma values (x,y,z) for type A
	double r0B[3];	//same, but type B
	double m0A[3];	//Molecule-frame dipole moment (x,y,z) for type A. Currently only the z component is used.
	double m0B[3];	//same, but type B
	double Solvent[3];	//Width, amplitude, and rmind for implicit solvent correction
	double muzA;	//Z component of molecule-frame dipole moment for type A
	double muzB;	//same, but type B
	double vdwA;	//Lennard-Jones epsilon for type A molecules
	double vdwB;	//same, but type B
	double stilleuler[3];	//Euler angles for non-rotating molecules
	double stillrot[3][3];	//Rotation matrix for non-rotating molecules
	double nndist;	//Nearest-neighbor distance for molecules (density^(-1/3))
	double ljcut;	//Lennard-Jones cutoff distance
	double escut;	//Electrostatics cutoff distance
	double rfcut;	//Reaction field cavity radius (generally equal to cnsts->escut) (SETUP)
	double tcut;	//Touch cutoff (SETUP)
	double nndmax;	//Approximate radius of nearest-neighbor shell	(SETUP)
	
	//precalculated touch, rho, and eps parameters for LJ interactions
	double pre_eps[2][2];	//Lennard-Jones epsilons
	double pre_sigma[2][2];	//Lennard-Jones sigma
	bool pre_touch[2][2];	//Control flags for whether or not to use touch
	//LDBR parameters. Can these be constant? (not currently used)
	double r;
	double fshift;
	double Vvdwboundary;
	double uBoundary;
	double rboundary;
	double uSigma;
	
};

