#pragma once
#include "Ellipse.h"

/*
 * © 2012, Lewis E. Johnson
 * Developed by Lewis E. Johnson and Robin Barnes, Robinson Group, University of Washington.
 * Released under University of Illinois/NCSA Open Source License. See LICENSE.txt for details.
 */
//Molecule class - stores molecule parameters, constructors, and functions for rotating and translating molecules. Fundamental
//class for systems being simulated; the entire ensemble is stored in an array of Molecules called BeOids -LEJ

class Molecule
{
public:
	const Ellipse* ellipse;	//Currently, molecules only contain one ellipse
	Vec3 center;	//Cordinates for the center of the molecule
    Mat33 rot;	//Rotation matrix to rotate the molecule from [0 0 1] to its lab frame position
	int still;	//Whether or not the molecule can move
	int rot_notrans;	//Whether or not the molecule can translate but not rotate
    int myindex;	//Index for an individual Molecule
	int mytype;	//Type for an individaul Molecule
	bool averageme;	//Whether or not a Molecule is included in averages for order parameter calculations + correlation functions
	double LastV;	//Previous potential energy for a molecule, used to revert energy if move rejected
	Molecule(const Ellipse&);
	Molecule(Molecule&);
	Molecule(const Ellipse& ellipse, int Still, int Rot_notrans, int mtype, bool avgme);	//new constructor (use this one)
public:
	~Molecule(void);
	double Rotate();	//Randomly rotate a molecule with the constraints of cnsts->maxrot
	double Translate();	//Randomly translate a molecule in a cubic box within the constraints of cnsts->maxtrans
	double Translate_Sphere();	//Same, but spherical box
	double Translate_Cylinder();	//Same, but cylinderical box
private:
	void SetEllipse(const Ellipse& e);
};

