/*
 * © 2012, Lewis E. Johnson
 * Developed by Lewis E. Johnson and Robin Barnes, Robinson Group, University of Washington.
 * Released under University of Illinois/NCSA Open Source License. See LICENSE.txt for details.
 *
 *  TotalV.h
 *  Created by ljohnson on 1/7/09.
 *	These functions are used to calculate total potential energy and determine when the total
 *	(or a significant subset of it) need to be recalculated, instead of just calculating the change in energy (as usual)
 */

#pragma once
#include "Molecule.h"
#include "OneStep.h"
#include "VmuE.h"

double VmuEtotal(Vec3* mus, Vec3 &Evec);	//Calculate total interaction energy between dipoles and poling field
void firststep(Molecule** BeOids, Vec3* mus, Vec3 &Evec, Vec3 &totalVs, bool skipmm);	//Calculate initial energy of entire system by (inefficently) calling OneStep Noids times. Called once per simulation.
bool setEandT(Vec3 &Evec, long step, double epsilon);	//Thermostat function and poling field update. Total poling energy must be recalculated if E0 changes
void recalcmm(Molecule** BeOids, Vec3* mus, Vec3 &totalVs);	//Recalculate total dipole_dipole interaction energy, needed if reaction field dielectric constant changes. Called once per cycle.
