/*
 * © 2012, Lewis E. Johnson
 * Developed by Lewis E. Johnson and Robin Barnes, Robinson Group, University of Washington.
 * Released under University of Illinois/NCSA Open Source License. See LICENSE.txt for details.
 *
 *  VecMat.cpp
 *  Created by ljohnson on 7/24/09. Last modified on 01/06/10
 *  Three-dimensional vector and matrix classes. Most functions are contained in the header; only non-inlined functions
 *	(mostly helper functions such as string conversion operators) are located here.
 */

#include "VecMat.h"

//String conversion functions
//Stringify a vector, with tabs between elements
std::string Vec3::V3Str() const
{
	std::stringstream converter;
	std::cout.precision(6);
	
	converter << std::fixed <<vec[0]<<"\t"<<vec[1]<<"\t"<<vec[2];
	
	std::string vecstring = converter.str();
	return vecstring;
}

//Stringify a matrix, with tabs between elements
std::string Mat33::M3Str()
{
	std::stringstream converter;
	std::cout.precision(6);
	

	converter << std::fixed <<mat[0][0]<<"\t"<<mat[0][1]<<"\t"<<mat[0][2]<<"\n";
	converter << std::fixed <<mat[1][0]<<"\t"<<mat[1][1]<<"\t"<<mat[1][2]<<"\n";
	converter << std::fixed <<mat[2][0]<<"\t"<<mat[2][1]<<"\t"<<mat[2][2];
	
	std::string matstring = converter.str();
	return matstring;
}

//Stringify a row of a matrix, with tabs between elements
std::string Mat33::M3RowStr(int i)
{
	std::stringstream converter;
	std::cout.precision(6);
	
	converter << std::fixed <<mat[i][0]<<"\t"<<mat[i][1]<<"\t"<<mat[i][2];
	
	std::string matstring = converter.str();
	return matstring;
}

//Gaussian elimination with partial pivoting for for solving Ax = b
//Used within M3LinSolve, but not inlined due to size
void Mat33::M3GEPP(Vec3 &x)
{
	//Declare variables. The code works in place, so no augmented matrix is constructed
	int i, j, k, pidx;
	double tempij;
	
	//Perform elimination
	for (i = 0; i < 3; i++)
	{
		//Find largest pivot, with index pidx
		pidx = i;
		for (j = i + 1; j < 3; j++)
		{
			if (fabs(mat[j][i]) > fabs(mat[pidx][i])) pidx = j; //fixed by AT
		}
		
		//Swap rows -- replace when figure out how to use class function within each other LEJ 01/07/10
		if (pidx != i)
		{
			for(j = 0; j < 3; j++)
			{
				tempij = mat[pidx][j];
				mat[pidx][j] = mat[i][j];
				mat[i][j] = tempij;
			}
			tempij = x.vec[pidx];
			x.vec[pidx] = x.vec[i];
			x.vec[i] = tempij;
		}
		
		//Eliminate
		for(k = i + 1; k < 3; k++)
		{
			x.vec[k] -= mat[k][i]/mat[i][i]*x.vec[i];
			for(j = 2; j >= i; j--)
			{
				mat[k][j] -= (mat[k][i]/mat[i][i])*mat[i][j];
			}
		}
	}
}
