#include "Vmumu.h"

/*
 * © 2012, Lewis E. Johnson
 * Developed by Lewis E. Johnson and Robin Barnes, Robinson Group, University of Washington.
 * Released under University of Illinois/NCSA Open Source License. See LICENSE.txt for details.
 */

//Dipole-dipole electrostatics module
//Input definitions
//	mymu - dipole moment of the active ellipsoid (kk)
//	themu - dipole moment of the molecule it is interacting with (i)
//	thermu - distance vector between molecules
//	r - norm of distance vector between molecules
//	r2 - square of norm of distance between molecules
//Return value is the dipole-dipole interaction energy for one pair of ellipsoids
//
//LEJ 12/30/2008, comments updated 01/13/2011

//Calculate pairwise dipole-dipole interactions for dipoles in the center of a molecule
double Vmumu(Vec3 &mymu, Vec3 &themu, Vec3 &thermu, double r, double r2)
{
	Cnsts* cnsts = GetCnsts();
	
	//Calculate the unit vector of the distance
	Vec3 r_hat = thermu/r;

	//Calculate all of the dot products so Vmumu can be calculated in a scalar equation
	double pdotr_hat = themu*r_hat;
	double mudotr_hat = mymu*r_hat;
	double mudotp = themu*mymu;

	//Calculate the dipole-dipole energy
	double Vmm = (mudotp-3.0*mudotr_hat*pdotr_hat)/(r*r2);
	
	//Calculate the reaction field correction if it is enabled (eps_ext != 1)
	Vmm -= mudotp*(cnsts->rfcorr);
	
	return Vmm;
}


