/*
 * © 2012, Lewis E. Johnson
 * Developed by Lewis E. Johnson and Robin Barnes, Robinson Group, University of Washington.
 * Released under University of Illinois/NCSA Open Source License. See LICENSE.txt for details.
 *
 *  get_positions.cpp
 *  Created by ljohnson on 10/9/08.
 *	Extracts positions from BeOids and creates a coordinate archive file. Usually only called at the end of a run,
 *	but may be called at arbitrary steps if appropriate flags are set.
 *	Inputs:
 *		BeOids - the ensemble of all molecules in the system
 *		Step - the current simulation step
 *		final - whether or not this is the end of the run
 */

#include "get_positions.h"

void get_positions(Molecule** BeOids, long Step, int final)
{
	//Get constants, etc.
	Cnsts* cnsts = GetCnsts();
	
	//Set file name string. Will be a number if an intermediate file, final if end of run, or exit if early termination
	string checkpoint;
	int counter = (int)(Step/((long)cnsts->grfreq));
	if (final == 1) checkpoint = "final";
	else if (final == 2) checkpoint = "exit";
	else checkpoint = int2str(counter);
	
	string thefilename = cnsts->fileout + int2str(cnsts->simnum) + "chk" + checkpoint + "coords.dat";
	FILE* f = fopen(thefilename.c_str(), "wa");
	
	//Print step at beginning of file, followed by semiaxes for ellipse types 0 and 1
	fprintf(f, "Step:	%li\n\n", Step);
	fprintf(f, "Types:\n0	%f	%f	%f\n",cnsts->r0A[0],cnsts->r0A[1],cnsts->r0A[2]);
	fprintf(f, "1	%f	%f	%f\n\n",cnsts->r0B[0],cnsts->r0B[1],cnsts->r0B[2]);
	
	//Print the centers and rotation matrices to a file. Documentation for the file format can be seen at
	//http://courses.washington.edu/bhrchem/visualizer.php
	for(int i=0; i<cnsts->n_oids; i++)
	{	
		fprintf(f, "%i	%i\n", i,BeOids[i]->mytype);
		fprintf(f, "%s\n%s\n\n\n", BeOids[i]->center.V3Str().c_str(), BeOids[i]->rot.M3Str().c_str());
	}
	
	fclose(f);
}

