/*
 * © 2012, Lewis E. Johnson
 * Developed by Lewis E. Johnson and Robin Barnes, Robinson Group, University of Washington.
 * Released under University of Illinois/NCSA Open Source License. See LICENSE.txt for details.
 *
 *  gofr.cpp
 *  Created by ljohnson on 10/27/08.
 *	This increasingly inaccurately named function, once only used to calculate the radial distribution function g(r)
 *	is now called to calculate several different correlation function, each of which is written to its own
 *	text file.
 */

#include "gofr.h"

//Initializes the files for correlation function calculations
void setup_gr()
{
	Cnsts *cnsts = GetCnsts();
	
	//g(r)
	string gfn = cnsts->fileout + int2str(cnsts->simnum) + "gr.dat";
	FILE* grf = fopen(gfn.c_str(), "wa");
	fprintf(grf, "Rmax	Bins	NumG(r)s\n");
	fprintf(grf, "%f	%i	%li\n\n", cnsts->rmax, cnsts->grinc, (cnsts->avg_steps/(long)cnsts->grfreq));
	fclose(grf);
	
	//nprofile(gamma)
	//Angular distribution histogram for nearest-neighbor shell.
	string ghn = cnsts->fileout + int2str(cnsts->simnum) + "gh.dat";
	FILE* ghf = fopen(ghn.c_str(), "wa");
	fprintf(ghf, "NNrmax	Bins	NumHs\n");
	fprintf(ghf, "%f	%i	%li\n\n", cnsts->nndmax, cnsts->grinc, (cnsts->avg_steps/(long)cnsts->grfreq));
	fclose(ghf);
	
	//cos(gamma)[cos(alpha)]
	//Angular distribution as a function of position around a central molecule
	string gnrn = cnsts->fileout + int2str(cnsts->simnum) + "gn.dat";
	FILE* gnf = fopen(gnrn.c_str(), "wa");
	fprintf(gnf, "NNrmax	Bins	NumHs\n");
	fprintf(gnf, "%f	%i	%li\n\n", cnsts->nndmax, cnsts->grinc, (cnsts->avg_steps/(long)cnsts->grfreq));
	fclose(gnf);
	
	//gmu_(r)
	//Dipole correlation function used for checking for ferroelectric domains
	string garn = cnsts->fileout + int2str(cnsts->simnum) + "ga.dat";
	FILE* gaf = fopen(garn.c_str(), "wa");
	fprintf(gaf, "Rmax	Bins	NumGa(r)s\n");
	fprintf(gaf, "%f	%i	%li\n\n", cnsts->rmax, cnsts->grinc, (cnsts->avg_steps/(long)cnsts->grfreq));
	fclose(gaf);
	
	//gmu2_(r)
	//Centrosymmetric dipole correlation function used for checking for LC domains
	string gasrn = cnsts->fileout + int2str(cnsts->simnum) + "gas.dat";
	FILE* gasf = fopen(gasrn.c_str(), "wa");
	fprintf(gasf, "Rmax	Rinc	NumGa2(r)s\n");
	fprintf(gasf, "%f	%i	%li\n\n", cnsts->rmax, cnsts->grinc, (cnsts->avg_steps/(long)cnsts->grfreq));
	fclose(gasf);
	
}


//Calculate g(r) and other correlation functions of choice
//Inputs - BeOids, the entire ensemble of molecules
void gofr(Molecule **BeOids)
{
	//Get constants, allocate memory, and generate useful values;
	Cnsts *cnsts = GetCnsts();
	string gfile = cnsts->fileout + int2str(cnsts->simnum) + "gr.dat";	//File for g(r)
	string ghfile = cnsts->fileout + int2str(cnsts->simnum) + "gh.dat";	//File for angular distribution histogram
	string gnfile = cnsts->fileout + int2str(cnsts->simnum) + "gn.dat";	//File for angle-angle distribution
	string gafile = cnsts->fileout + int2str(cnsts->simnum) + "ga.dat";	//File for dipole correlation function
	string gasfile = cnsts->fileout + int2str(cnsts->simnum) + "gas.dat";	//File for centrosymmetric dipole correlation function
	
	int n,k,l,y,y2;
	int Noids = cnsts->n_oids;
	double r, gr, pbdtemp, cg, ca, ghtemp, gntemp;
	double Rmax = cnsts->rmax;
	double nnmax = cnsts->nndmax;		
	double dr = cnsts->refdr;
	double N = (double)cnsts->n_avg;
	double N2 = N*(N-1);
	double gatemp, gastemp;
	
	Vec3 rij, rij_unit;
	
	double* h = new double[cnsts->grinc]; //radial histogram
	double* hg = new double[cnsts->grinc]; //neighbor shell & internal angle histogram
	double* ha = new double [cnsts->grinc]; //cos internal angle as a function of r histogram
	double* has = new double[cnsts->grinc]; //cos dipole/position angle as a function of r cumsum
	double* hah = new double[cnsts->grinc]; //cos dipole/position angle a function of r histogram
	double* ha2 = new double[cnsts->grinc]; //cos^2 internal angle as a function of r histogram
	
	//Zero the arrays that will be used to store the histograms
	for (n = 0; n < cnsts->grinc; n++)
	{
		h[n] = 0.0;
		ha[n] = 0.0;
		has[n] = 0.0;
		hah[n] = 0.0;
		hg[n] = 0.0;
		ha2[n] = 0.0;
	}

	
	//cycle through all (oids i, j>i) to calculate g(r)
	for(int i = 0; i < Noids; i++)
	{
		if (BeOids[i]->averageme == 1)
		{

			for(int j = i + 1; j < Noids; j++)
			{
				if (BeOids[j]->averageme == 1)
				{
					rij = BeOids[j]->center - BeOids[i]->center;
			
					//Make PBC adjustments if needed
					for (k = 0; k < 3; k++)
					{
						if (cnsts->PBCs[k] == 1)
						{
							pbdtemp = rij.vec[k]/(cnsts->boxlength[k]);
							rij.vec[k] -= cnsts->boxlength[k]*qround(pbdtemp);
						}
					}
					
					//Get unit vector between j and i
					r = rij.V3Norm();
					rij_unit = rij/r;
			
					//Calculate internal angles
					cg=0.0; ca = 0.0;
					for(k = 0; k < 3; k++)
					{
						cg += (BeOids[i]->rot.mat[k][2])*(BeOids[j]->rot.mat[k][2]);	//cos(gamma) -> angle between dipoles i and j
						ca += rij_unit.vec[k]*(BeOids[i]->rot.mat[k][2]);	//cos(alpha) -> angle between dipole i and vector between centers i and j
					}

					//generate histogram
					if (r < Rmax)
					{
						l = (int)(r/dr); //assign to r slice
						h[l] +=1.0; //radial histogram
						ha[l] += cg;  //internal angle sum
						ha2[l] += cg*cg;
						
						if (r < nnmax)
						{
							y = (int)fastfloor((0.5*(cg+1.0)*cnsts->grinc)); //assign angular slice for cos(gamma)
							y2 = (int)fastfloor((0.5*(ca+1.0)*cnsts->grinc)); //assign to angular slice for cos(alpha)
							hg[y] +=1.0; //internal angle histogram
							has[y2] += cg; //Positional/orientational correlation.
							hah[y2] += 1.0;
						}
					}
				}
			}
		}
	}

	
	//Prepare files for output
	FILE * g = fopen(gfile.c_str(), "a");
	FILE * gh = fopen(ghfile.c_str(), "a");
	FILE * ga = fopen(gafile.c_str(), "a");
	FILE * gn = fopen(gnfile.c_str(), "a");
	FILE * gas = fopen(gasfile.c_str(), "a");
	
	
	//normalize to generate g(r). Divides by 2 instead of 4 since only half of the mx is used.
	//also calculate <cos(gamma)>(r) since it uses same format and histogram as g(r)
	for(n = 0; n < cnsts->grinc; n++)
	{
		//g(r)
		double r2dr = qpwr(dr*(n+1), 2)*dr;
		gr = h[n]*cnsts->V/(2.0*pi*N2*r2dr);
		fprintf(g, "%f	", gr);
		
		//gmu(r)
		gatemp = ha[n];
		fprintf(ga, "%f	",gatemp);
		
		//g2mu(r)
		gastemp = ha2[n];
		fprintf(gas,"%f	",gastemp);
		
		//gamma_hist
		ghtemp = hg[n];
		fprintf(gh,"%f	",ghtemp);
		
		//cos(gamma)[cos(alpha)]
		if (hah[n] != 0) gntemp = has[n]/hah[n];
		else gntemp = 0.0;
		fprintf(gn,"%f	",gntemp);
	}
	
	fprintf(g, "\n");
	fprintf(ga, "\n");
	fprintf(gas,"\n");
	fprintf(gh, "\n");
	fprintf(gn, "\n");
	
	
	//clean up
	delete h;
	delete hg;
	delete ha;
	delete hah;
	delete has;
	delete ha2;

	fclose(g);
	fclose(ga);
	fclose(gas);
	fclose(gh);
	fclose(gn);
}

