/*
 * © 2012, Lewis E. Johnson
 * Developed by Lewis E. Johnson and Robin Barnes, Robinson Group, University of Washington.
 * Released under University of Illinois/NCSA Open Source License. See LICENSE.txt for details.
 */

// utility_functions.cpp : source file that includes standard library includes + some basic math and text handling functions
// The old linear algebra functions have been removed and replaced with new ones located in VecMat.h - LEJ 01/08/10

#include "utility_functions.h"

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
//#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2()
//initialize with negative integer idum.
{
	Cnsts* cnsts = GetCnsts();
	long idum = *cnsts->idum;
	int j;
	long k;
	static long idum2=123456789;
	static long iy=0;
	static long iv[NTAB];
	double temp;

	if (idum <= 0) {
		if (-(idum) < 1) idum=1;
		else idum = -(idum);
		idum2=(idum);
		for (j=NTAB+7;j>=0;j--) {
			k=(idum)/IQ1;
			idum=IA1*(idum-k*IQ1)-k*IR1;
			if (idum < 0) idum += IM1;
			if (j < NTAB) iv[j] = idum;
		}
		iy=iv[0];
	}
    k=(idum)/IQ1;
	idum=IA1*(idum-k*IQ1)-k*IR1;
    if (idum < 0) idum += IM1;
	k=idum2/IQ2;
    idum2=IA2*(idum2-k*IQ2)-k*IR2;
	if (idum2 < 0) idum2 += IM2;
	j=iy/NDIV;
	iy=iv[j]-idum2;
	iv[j] = idum;
	if (iy < 1) iy += IMM1;
	*cnsts->idum = idum;
	if ((temp= (double)AM*iy) > RNMX) return (double)RNMX;
	else return temp;
}


Cnsts global_Cnsts;
Cnsts* GetCnsts()
{
return &global_Cnsts;
}

string int2str(int A)
{
	stringstream converter;
	converter << A;
	return converter.str();
}

//Averages a 1D array of length numel
double average(double *A, int numel)
{
	double temp = 0;
	
	for(int i = 0; i<numel; i++)
	{
		temp += A[i];
	}
	
	double avg = temp/(double)numel;
	return avg;
}

//Averages a range of a 1D array
double average_range(double *A, int first, int last)
{
	double temp = 0;
	
	for(int i = first; i<last; i++)
	{
		temp += A[i];
	}
	
	double avg = temp/(double)(last-first);
	return avg;
}

//Averages a 2D array
void average2x(double **A, double *B, int numelx, int numely)
{
	double temp;
	
	for(int i = 0; i<numely; i++)
	{
		temp = 0;
		B[i] = 0;
		
		for(int j = 0; j<numelx; j++)
		{
			temp +=A[i][j];
		}
		
		B[i] += temp/(double)numelx;
	}
}

//Averages a range of a 2D array
void average_range2x(double **A, double *B, int firstx, int lastx, int firsty, int lasty)
{
	double temp;
	
	for(int i = firsty; i<lasty; i++)
	{
		temp = 0;
		B[i] = 0;
		
		for(int j = firstx; j<lastx; j++)
		{
			temp +=A[i][j];
		}
		
		B[i] += temp/(double)(lastx-firstx);
	}
}

//Sums an array of ints
int totalcounti(int *A, int numel)
{
	int tc = 0;
	
	for(int i = 0; i<numel; i++)
	{
		tc += A[i];
	}
	
	return tc;
}

//sums an array of doubles
double totalcountd(double *A, int numel)
{
	double tc = 0;
	
	for(int i = 0; i<numel; i++)
	{
		tc += A[i];
	}
	
	return tc;
}

//Creates a tensor from two vectors
void TensorProduct3(Vec3 &A, Vec3 &B, Mat33 &C)
{
	for(int i = 0; i<3; i++)
	{
		C.mat[i][0] = A.vec[i]*B.vec[0];
		C.mat[i][1] = A.vec[i]*B.vec[1];
		C.mat[i][2] = A.vec[i]*B.vec[2];
	}			
}

//Array (element-wise) multiply C[i] = A[i]*B[i] for 3-vectors
void Vec3Amult(Vec3 &A, Vec3 &B, Vec3 &C)
{
	C.vec[0] = A.vec[0]*B.vec[0];
	C.vec[1] = A.vec[1]*B.vec[1];
	C.vec[2] = A.vec[2]*B.vec[2];
}

//Array (element-wise) divide C[i] = A[i]/B[i] for 3-vectors
void Vec3Adiv(Vec3 &A, Vec3 &B, Vec3 &C)
{
	C.vec[0] = A.vec[0]/B.vec[0];
	C.vec[1] = A.vec[1]/B.vec[1];
	C.vec[2] = A.vec[2]/B.vec[2];
}

//Advance line function -- moves string pointer to after next \n
void advanceline(fstream &file, int nline)
{
	int i;
	char spt;
	
	for(i=0; i<nline;i++)
	{
		do file.get(spt);
		while (spt != '\n');
	}
}

//Get tab-delimited int function -- reads in chars until encounters delimiter, then converts to int
int gettdint(fstream &file)
{
	char ch;
	string buffer;
	while (file.get(ch) && ch != '\t' && ch != '\n')
	{
		buffer += ch;
	}
	
	int i = atoi(buffer.c_str());
	return i;
}

//Get tab-delimited double function -- reads in chars until encounters delimiter, then converts to double
double gettddoub(fstream &file)
{
	char ch;
	string buffer;
	while (file.get(ch) && ch != '\t' && ch != '\n')
	{
		buffer += ch;
	}
	
	double d = atof(buffer.c_str());
	return d;
}

