/*
 * © 2012, Lewis E. Johnson
 * Developed by Lewis E. Johnson and Robin Barnes, Robinson Group, University of Washington.
 * Released under University of Illinois/NCSA Open Source License. See LICENSE.txt for details.
 */

// utility_functions.h : header file containing standard library includes
// and inlined versions of a few utility functions

#pragma once

//Standard library
#include <cstdio>
#include <ctime>
#include <string>
#include <cmath>
#include <cstdlib>
#include <iostream>
#include <sstream>
#include <fstream>

//Custom Classes
#include "Cnsts.h"
#include "VecMat.h"

//Define constants here
#define BUILD "051612" //Date of current build. Do not update if only simulation parameters are changed.
#define pi 3.14159265
#define EPS 1.2e-7

Cnsts* GetCnsts();

//Global functions
//Placeholder linear algebra functions have been removed; their replacements are in VecMat.h LEJ 01/08/10

//The Robin Block
double ran2();	//Random number generator

//The Lewis block
string int2str(int A); //Convert an integer to a string
double average(double *A, int numel);	//Calculate the mean of an array of length numel
double average_range(double *A, int first, int last);	//Calculate the mean of part of an array
void average2x(double **A, double *B, int numelx, int numely);	//Same as above, but 2D
void average_range2x(double **A, double *B, int firstx, int lastx, int firsty, int lasty);	//Same as above, but 2D
int totalcounti(int *A, int numel);	//Sum of an integer array
double totalcountd(double *A, int numel);	//Sum of a double array
void TensorProduct3(Vec3 &A, Vec3 &B, Mat33 &C); //REPLACE - see VecMat.h for details
void Vec3Amult(Vec3 &A, Vec3 &B, Vec3 &C);	//Multiply two Vec3s as if they are normal arrays, instead of doing a dot product
void Vec3Adiv(Vec3 &A, Vec3 &B, Vec3 &C);	//Divide two Vec3s as if they are normal arrays
void advanceline(fstream &file, int nline);	//File read utility for advancing a line
int gettdint(fstream &file);	//Extracts tab-delimited integers from a line of a file
double gettddoub(fstream &file);	//Extracts tab-delimited doubles from a line of a file

//The inlined Lewis block
//Optimized floor algorithm suggested by DTrace
inline double fastfloor(double f)
{
	register double twoTo52 = 4503599627370496.0;
	double c = ( f >= 0.0 ? -twoTo52 : twoTo52 );
	double result = (f - c) + c;
	if( f < result ) result -= 1.0;
	return result;
}

//Fast rounding algorithm using fastfloor
inline double qround(double A)
{
	double rA = fastfloor(A+0.5);
	return rA;
}

//Fast power algorithm that only works for positive integers
inline double qpwr(double a, int b)
{
	double c = 1;
	for (int i = 0; i < b; i++) c *= a;
	return c;
}

//Maximum value for an array of doubles
inline double maxd(double *A, int l)
{
	int i;
	double Max = A[0];
	for (i = 1; i < l; i++)
	{
		if (A[i] > Max) Max = A[i];
	}
	return Max;
}

//Minimum value for an array of doubles
inline double mind(double *A, int l)
{
	int i;
	double Min = A[0];
	for (i = 1; i < l; i++)
	{
		if (A[i] < Min) Min = A[i];
	}
	return Min;
}


